from tkinter import *
from PIL import Image, ImageTk
import numpy as np
import random
import matplotlib.pyplot as plt
import os
from scipy.integrate import odeint


class Application:
    
    def __init__(self):
        self.anim=0
        self.viral=20
        self.period=7
        self.mortal=0
        self.jour=0
        
        #creer la fenetre
        self.window = Tk()
        self.window.title("SIR model")
        self.window.geometry("1080x720")
        self.window.resizable(width=0, height=0)
        self.window.config(background='ivory')
        self.window.iconbitmap("Polytech_Angers.ico")

        #creer les frames
        self.frame1 = Frame(self.window)
        self.frame2 = Frame(self.window,background='ivory')

        #creer image
        self.img = Image.open("carte.png")
        self.image = ImageTk.PhotoImage(self.img.resize((700,760),4))
        self.canvas = Canvas(self.frame1,width=700,height=760,bg='gray')
        self.item = self.canvas.create_image(350,380,image=self.image)
        self.alea()
        self.canvas.pack()
        
        self.canvas.bind("<Button-1>", self.click_gauche)
        
        #zone de texte
        self.label = Label(self.frame2,font = ('Sans','20','bold'),fg='#009CDD',background='ivory')
        self.label.configure(text = "No simulation in progress")
        self.label.pack(expand=YES)

        self.label4 = Label(self.frame2,font = ('Sans','20','bold'),fg='#009CDD',background='ivory')
        self.label4.configure(text = self.jour)
        self.label4.pack(expand=YES)

        #creer les boutons
        self.bouton1=Button(self.frame2,text='Starting the simulation',command=self.sir,height=2,width=25,fg='green',font = ('Sans','20','bold'),background='ivory').pack(expand=YES)
        self.bouton2=Button(self.frame2,text='Pause',command=self.pause,height=2,width=25,fg='black',font = ('Sans','20','bold'),background='ivory').pack(expand=YES)
        self.bouton3=Button(self.frame2,text='Reset',command=self.reset,height=2,width=25,fg='red',font = ('Sans','20','bold'),background='ivory').pack(expand=YES)

        
        self.label13 = Label(self.frame2,background='ivory')
        self.label13.configure(text = "Coefficient of disease transmission (in %) :")
        self.label13.pack(expand=YES)
        self.bouton5=Button(self.frame2,text='+',command=self.viralp,fg='green',background='ivory').pack(expand=YES)
        self.label2 = Label(self.frame2,background='ivory')
        self.label2.configure(text = self.viral)
        self.label2.pack(expand=YES)
        self.bouton6=Button(self.frame2,text='-',command=self.viralm,fg='red',background='ivory').pack(expand=YES)


        self.label13 = Label(self.frame2,background='ivory')
        self.label13.configure(text = "Period of transmissibility of the disease (in days) :")
        self.label13.pack(expand=YES)
        self.bouton7=Button(self.frame2,text='+',command=self.periodp,fg='green',background='ivory').pack(expand=YES)
        self.label3 = Label(self.frame2,background='ivory')
        self.label3.configure(text = self.period)
        self.label3.pack(expand=YES)
        self.bouton8=Button(self.frame2,text='-',command=self.periodm,fg='red',background='ivory').pack(expand=YES)

        self.label23 = Label(self.frame2,background='ivory')
        self.label23.configure(text = "Mortality of the disease (in %) :")
        self.label23.pack(expand=YES)
        self.bouton27=Button(self.frame2,text='+',command=self.mortalp,fg='green',background='ivory').pack(expand=YES)
        self.label23 = Label(self.frame2,background='ivory')
        self.label23.configure(text = self.mortal)
        self.label23.pack(expand=YES)
        self.bouton28=Button(self.frame2,text='-',command=self.mortalm,fg='red',background='ivory').pack(expand=YES)

        self.bouton4=Button(self.frame2,text='Quit',command=lambda :self.quitter(),height=2,width=25,fg='#004A75',font = ('Sans','20','bold'),background='ivory').pack(expand=YES)
    
        #afficher les frames
        self.frame1.pack(side=LEFT)
        self.frame2.pack(side=RIGHT)
    
        

    def viralp(self):
        if self.viral <= 90:
            self.viral = self.viral+10
            self.label2.configure(text = self.viral)

    def viralm(self):
        if self.viral >= 20:
            self.viral = self.viral-10
            self.label2.configure(text = self.viral)

    def mortalp(self):
        if self.mortal <= 90:
            self.mortal = self.mortal+10
            self.label23.configure(text = self.mortal)

    def mortalm(self):
        if self.mortal >= 10:
            self.mortal = self.mortal-10
            self.label23.configure(text = self.mortal)

    def periodp(self):
        if self.period < 99:
            self.period = self.period+1
            self.label3.configure(text = self.period)

    def periodm(self):
        if self.period > 0:
            self.period = self.period-1
            self.label3.configure(text = self.period)


            
    def sir(self):
        if self.anim == 0 :
            self.anim = 1
            self.jour = 0

            self.nbsain = 0
            self.tabsain = []
            
            self.nbinfecte = 0
            self.tabinfecte = []

            self.nbretabli = 0
            self.tabretabli = []

            self.tabtotal = []

            #tableau pour incubation
            self.incub = np.zeros((self.imgnp.shape[0],self.imgnp.shape[1]))
            for i in range(self.imgnp.shape[0]):
                for j in range(self.imgnp.shape[1]):
                    if self.imgnp[i,j][0] == 250:
                        self.incub[i,j]+=1
            self.poll()
            self.label.configure(text = "Simulation in progress: SIR model")


            
            
    def poll (self) :
        if self.anim == 1:
            self.simu_sir()
            self.jour =self.jour+1
            self.tabsain.append(self.nbsain)
            self.tabinfecte.append(self.nbinfecte)
            self.tabretabli.append(self.nbretabli)
            self.tabtotal.append(self.nbsain+self.nbinfecte+self.nbretabli)
            
            if self.nbinfecte == 0:
                plt.plot(self.tabsain,"g",label="Suceptible")
                plt.plot(self.tabinfecte,"r",label="Infected")
                plt.plot(self.tabretabli,"k",label="Recovered")
                plt.plot(self.tabtotal,"b",label="Total population")
                plt.xlabel("Days")
                plt.ylabel("Cells number")
                plt.title("Simulation result")
                plt.legend()

                #Modele mathématique
                t = np.arange(0,self.jour+1)
                y0 = self.tabsain[0] , self.tabinfecte[0] , self.tabretabli[0]
                N = self.tabsain[0] + self.tabinfecte[0] + self.tabretabli[0]

                def deriv(y, t, N, beta, gamma ,mu):
                    S, I, R = y
                    dSdt = -beta * S * I / N 
                    dIdt = beta * S * I / N -  I/gamma - I*mu
                    dRdt = I/gamma
                    return dSdt, dIdt, dRdt

                ret = odeint(deriv, y0, t, args=(N, self.viral/100, self.period, self.mortal/100))
                S, I, R = ret.T

                pop = []
                for i in range(len(S)):
                    pop.append(S[i]+I[i]+R[i])

                plt.plot(S,'g--')
                plt.plot(I,'r--')
                plt.plot(R,'k--')
                plt.plot(pop,'b--')

                plt.show()
                self.anim = 0


            self.label4.configure(text = self.jour)
            self.window.after(200,self.poll)

        
            
            
    def pause(self):
        self.anim = 0
        self.label.configure(text = "Simulation in pause")

        
            
    def reset(self):
        self.img = Image.open("carte.png")
        self.image = ImageTk.PhotoImage(self.img.resize((700,760),4))
        self.item = self.canvas.create_image(350,380,image=self.image)
        self.alea()
        self.label.configure(text = "Map reset")
        
   

    def simu_sir(self):
        self.nbsain=0
        self.nbinfecte=0
        self.nbretabli=0
        self.nbmort=0
        #convertir en tableau
        self.imgnp = np.array(self.img)
        self.imgnp2 = np.copy(self.imgnp)
        
        #traiter image
        for i in range(self.imgnp.shape[0]):
            for j in range(self.imgnp.shape[1]):
                self.c=0
                if self.imgnp[i,j][1] == 250 :
                    self.nbsain=self.nbsain+1
                    
                    if self.imgnp[i+1, j+1][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i-1, j-1][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i+1, j][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i-1, j][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i, j+1][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i, j-1][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i-1, j+1][0] == 250:
                        self.c=self.c+1
                        
                    if self.imgnp[i+1, j-1][0] == 250:
                        self.c=self.c+1
                   
                    if self.c > 0 and random.random()<self.viral/100:
                        self.imgnp2[i, j]=(250,0,0,255)
                        
                if self.imgnp[i,j][0] == 250:
                    if random.random()<self.mortal/100:
                        self.imgnp2[i, j]=(0,0,0,255)
                    else:
                        self.incub[i,j]+=1
                    
                    self.nbinfecte=self.nbinfecte+1
                    if self.incub[i,j]>=self.period:
                        self.imgnp2[i, j]=(127,143,166,255)

                if self.imgnp[i,j][0] == 127:
                    self.nbretabli=self.nbretabli+1
                   
                        
        for k in range(self.imgnp2.shape[0]):
            for l in range(self.imgnp2.shape[1]):
                
                if self.imgnp2[k,l][1] == 250 or self.imgnp2[k,l][0] == 250 or self.imgnp2[k,l][0] == 127:
                    self.x=k+random.randint(-1, 1)
                    self.y=l+random.randint(-1, 1)
                    
                    if self.imgnp2[self.x,self.y][0] == 78 :
                        self.imgnp2[self.x,self.y]=self.imgnp2[k,l]
                        self.imgnp2[k,l]=(78,191,239,255)
                        
                        self.incub[self.x,self.y]=self.incub[k,l]
                        self.incub[k,l]=0
                    
        #convertir en image
        self.imgpil = Image.fromarray(self.imgnp2)
        self.img = self.imgpil
            
        #integrer l'imafe au canvas
        self.image = ImageTk.PhotoImage(self.imgpil.resize((700,760),4))
        self.item = self.canvas.create_image(350,380,image=self.image)


    def click_gauche(self,evt):
        #convertir en tableau
        self.imgnp = np.array(self.img)
        self.imgnp2 = np.copy(self.imgnp)
        
        #traiter image
        if self.imgnp[evt.y//10,evt.x//10][0] == 78 or self.imgnp[evt.y//10,evt.x//10][1] == 250:
            self.imgnp2[evt.y//10,evt.x//10] = (250,0,0,255)
            
        if self.imgnp[evt.y//10,evt.x//10][0] == 250:
            self.imgnp2[evt.y//10,evt.x//10] = (78,191,239,255)
        
        #convertir en image
        self.imgpil = Image.fromarray(self.imgnp2)
        self.img = self.imgpil
            
        #integrer l'imafe au canvas
        self.image = ImageTk.PhotoImage(self.imgpil.resize((700,760),4))
        self.item = self.canvas.create_image(350,380,image=self.image)

        

    def alea(self):
        #convertir en tableau
        self.imgnp = np.array(self.img)
        self.imgnp2 = np.copy(self.imgnp)
        
        #traiter image
        
        for i in range(self.imgnp.shape[0]):
            for j in range(self.imgnp.shape[1]):
                if self.imgnp[i,j][0] == 78 and self.imgnp[i,j][0]!=250:
                    if random.randint(1, 4)==1:
                        self.imgnp2[i, j]=(0,250,0,255)

        #convertir en image
        self.imgpil = Image.fromarray(self.imgnp2)
        self.img = self.imgpil
            
        #integrer l'imafe au canvas
        self.image = ImageTk.PhotoImage(self.imgpil.resize((700,760),4))
        self.item = self.canvas.create_image(350,380,image=self.image)

    def quitter(self):
        self.window.destroy()

        
if __name__ == "__main__":
    app=Application()
    app.window.mainloop()
    

